@@ -30,8 +30,18 @@ class CephCluster(object):
 
         self.error = False
         self.error_msg = ''
-        self.cluster = rados.Rados(conffile=settings.config.cephconf,
-                                   name=settings.config.cluster_client_name)
+        self.cluster = None
+
+        conf = settings.config.cephconf
+
+        try:
+            self.cluster = rados.Rados(conffile=conf,
+                                       name=settings.config.cluster_client_name)
+        except rados.Error as err:
+            self.error = True
+            self.error_msg = "Invaid cluster_client_name or setting in {} - {}".format(conf, err)
+            return
+
         try:
             self.cluster.connect()
         except rados.Error as err:
@@ -39,7 +49,8 @@ class CephCluster(object):
             self.error_msg = "Unable to connect to the cluster (keyring missing?) - {}".format(err)
 
     def __del__(self):
-        self.cluster.shutdown()
+        if self.cluster:
+            self.cluster.shutdown()
 
     def shutdown(self):
         self.cluster.shutdown()
@@ -64,9 +75,11 @@ class Config(object):
 
     lock_time_limit = 30
 
-    def __init__(self, logger, cfg_name='gateway.conf', pool=None):
+    def __init__(self, logger, cfg_name=None, pool=None):
         self.logger = logger
         self.config_name = cfg_name
+        if self.config_name is None:
+            self.config_name = settings.config.gateway_conf
         if pool is None:
             pool = settings.config.pool
         self.pool = pool
@@ -186,52 +199,58 @@ class Config(object):
             self.update_item("version", element_name=None, element_value=3)
 
         if self.config['version'] == 3:
-            iqn = self.config['gateways']['iqn']
+            iqn = self.config['gateways'].get('iqn', None)
             gateways = {}
             portals = {}
-            for host, gateway_v3 in self.config['gateways'].items():
-                if isinstance(gateway_v3, dict):
-                    portal = gateway_v3
-                    portal.pop('iqn')
-                    active_luns = portal.pop('active_luns')
-                    updated = portal.pop('updated', None)
-                    created = portal.pop('created', None)
-                    gateway = {
-                        'active_luns': active_luns
-                    }
-                    if created:
-                        gateway['created'] = created
-                    if updated:
-                        gateway['updated'] = updated
-                    gateways[host] = gateway
-                    portals[host] = portal
-            for _, client in self.config['clients'].items():
-                client.pop('created', None)
-                client.pop('updated', None)
-                client['auth']['chap_mutual'] = ''
-            for _, group in self.config['groups'].items():
-                group.pop('created', None)
-                group.pop('updated', None)
-            target = {
-                'disks': list(self.config['disks'].keys()),
-                'clients': self.config['clients'],
-                'portals': portals,
-                'groups': self.config['groups'],
-                'controls': self.config.get('controls', {}),
-                'ip_list': self.config['gateways']['ip_list']
-            }
+
+            self.add_item("targets", None, {})
             self.add_item('discovery_auth', None, {
                 'chap': '',
                 'chap_mutual': ''
             })
-            self.add_item("targets", None, {})
-            self.add_item("targets", iqn, target)
-            self.update_item("targets", iqn, target)
+
+            if iqn:
+                for host, gateway_v3 in self.config['gateways'].items():
+                    if isinstance(gateway_v3, dict):
+                        portal = gateway_v3
+                        portal.pop('iqn')
+                        active_luns = portal.pop('active_luns')
+                        updated = portal.pop('updated', None)
+                        created = portal.pop('created', None)
+                        gateway = {
+                            'active_luns': active_luns
+                        }
+                        if created:
+                            gateway['created'] = created
+                        if updated:
+                            gateway['updated'] = updated
+                        gateways[host] = gateway
+                        portals[host] = portal
+                for _, client in self.config['clients'].items():
+                    client.pop('created', None)
+                    client.pop('updated', None)
+                    client['auth']['chap_mutual'] = ''
+                for _, group in self.config['groups'].items():
+                    group.pop('created', None)
+                    group.pop('updated', None)
+                target = {
+                    'disks': list(self.config['disks'].keys()),
+                    'clients': self.config['clients'],
+                    'portals': portals,
+                    'groups': self.config['groups'],
+                    'controls': self.config.get('controls', {}),
+                    'ip_list': self.config['gateways']['ip_list']
+                }
+                self.add_item("targets", iqn, target)
+                self.update_item("targets", iqn, target)
+
+            self.update_item("gateways", None, gateways)
+
             if 'controls' in self.config:
                 self.del_item('controls', None)
             self.del_item('clients', None)
             self.del_item('groups', None)
-            self.update_item("gateways", None, gateways)
+
             self.update_item("version", None, 4)
 
         if self.config['version'] == 4:
@@ -600,7 +619,7 @@ class Config(object):
                     del current_config[txn.type]
             else:
                 self.error = True
-                self.error_msg = "Unknown transaction type ({}} encountered in " \
+                self.error_msg = "Unknown transaction type ({}) encountered in " \
                                  "_commit_rbd".format(txn.action)
 
         if not self.error:
