@@ -562,7 +562,7 @@ class Client(UINode):
         mapped_disks = [mapped_disk.name
                         for mapped_disk in self.parent.parent.target_disks.children]
         if disk not in mapped_disks:
-            rc = self.parent.parent.target_disks.add_disk(disk, None)
+            rc = self.parent.parent.target_disks.add_disk(disk, None, None)
             if rc == 0:
                 self.logger.debug("disk auto-map successful")
             else:
