@@ -153,7 +153,7 @@ class Disks(UIGroup):
         for child in children:
             self.remove_child(child)
 
-    def ui_command_attach(self, pool=None, image=None, backstore=None):
+    def ui_command_attach(self, pool=None, image=None, backstore=None, wwn=None):
         """
         Assign a previously created RBD image to the gateway(s)
 
@@ -187,9 +187,10 @@ class Disks(UIGroup):
 
         self.logger.debug("CMD: /disks/ attach pool={} "
                           "image={}".format(pool, image))
-        self.create_disk(pool=pool, image=image, create_image=False, backstore=backstore)
+        self.create_disk(pool=pool, image=image, create_image=False, backstore=backstore, wwn=wwn)
 
-    def ui_command_create(self, pool=None, image=None, size=None, backstore=None, count=1):
+    def ui_command_create(self, pool=None, image=None, size=None, backstore=None, wwn=None,
+                          count=1):
         """
         Create a RBD image and assign to the gateway(s).
 
@@ -208,6 +209,7 @@ class Disks(UIGroup):
         size  : integer, suffixed by the allocation unit - either m/M, g/G or
                 t/T representing the MB/GB/TB [1]
         backstore  : lio backstore
+        wwn   : unit serial number
         count : integer (default is 1)[2]. If the request provides a count=<n>
                 parameter the image name will be used as a prefix, and the count
                 used as a suffix to create multiple images from the same request.
@@ -258,7 +260,8 @@ class Disks(UIGroup):
         self.logger.debug("CMD: /disks/ create pool={} "
                           "image={} size={} "
                           "count={} ".format(pool, image, size, count))
-        self.create_disk(pool=pool, image=image, size=size, count=count, backstore=backstore)
+        self.create_disk(pool=pool, image=image, size=size, count=count, backstore=backstore,
+                         wwn=wwn)
 
     def _valid_pool(self, pool=None):
         """
@@ -283,7 +286,7 @@ class Disks(UIGroup):
         return False
 
     def create_disk(self, pool=None, image=None, size=None, count=1,
-                    parent=None, create_image=True, backstore=None):
+                    parent=None, create_image=True, backstore=None, wwn=None):
 
         rc = 0
 
@@ -307,7 +310,7 @@ class Disks(UIGroup):
         api_vars = {'pool': pool, 'owner': local_gw,
                     'count': count, 'mode': 'create',
                     'create_image': 'true' if create_image else 'false',
-                    'backstore': backstore}
+                    'backstore': backstore, 'wwn': wwn}
         if size:
             api_vars['size'] = size.upper()
 
@@ -624,8 +627,9 @@ class DiskPool(UIGroup):
 class Disk(UINode):
 
     display_attributes = ["image", "ceph_cluster", "pool", "wwn", "size_h",
-                          "feature_list", "snapshots", "owner", "backstore",
-                          "backstore_object_name", "control_values"]
+                          "feature_list", "snapshots", "owner", "lock_owner",
+                          "state", "backstore", "backstore_object_name",
+                          "control_values"]
 
     def __init__(self, parent, image_id, image_config, size=None,
                  features=None, snapshots=None):
@@ -661,6 +665,7 @@ class Disk(UINode):
             self.parent.parent.disk_lookup[image_id] = self
 
         self._apply_config(image_config)
+        self._apply_status()
 
         if not size:
             # Size/features are not stored in the config, since it can be changed
@@ -681,6 +686,34 @@ class Disk(UINode):
         disk_map[self.image_id]['size'] = self.size
         disk_map[self.image_id]['size_h'] = self.size_h
 
+    def _apply_status(self):
+        disk_api = ('{}://localhost:{}/api/'
+                    'disk/{}'.format(self.http_mode,
+                                     settings.config.api_port, self.image_id))
+        self.logger.debug("disk GET status for {}".format(self.image_id))
+        api = APIRequest(disk_api)
+        api.get()
+
+        # set both the 'lock_owner' and 'state' to Unknown as default in
+        # case if the api response fails the gwcli command will fail too
+        self.__setattr__('lock_owner', 'Unknown')
+        self.__setattr__('state', 'Unknown')
+
+        if api.response.status_code == 200:
+            info = api.response.json()
+            status = info.get("status")
+
+            if status is None:
+                return
+
+            state = status.get('state')
+            if (state):
+                self.__setattr__('state', state)
+
+            owner = status.get('lock_owner')
+            if (owner):
+                self.__setattr__('lock_owner', owner)
+
     def _apply_config(self, image_config):
         # set the remaining attributes based on the fields in the dict
         disk_map = self.parent.parent.disk_info
@@ -697,9 +730,28 @@ class Disk(UINode):
         if not self.exists:
             return 'NOT FOUND', False
 
-        msg = [self.image_id, "({})".format(self.size_h)]
+        status = True
 
-        return " ".join(msg), True
+        disk_api = ('{}://localhost:{}/api/'
+                    'disk/{}'.format(self.http_mode, settings.config.api_port,
+                                     self.image_id))
+
+        self.logger.debug("disk GET status for {}".format(self.image_id))
+        api = APIRequest(disk_api)
+        api.get()
+
+        state = "State unknown"
+        if api.response.status_code == 200:
+            info = api.response.json()
+            disk_status = info.get("status")
+            if disk_status:
+                state = disk_status.get("state")
+                if state != "Online":
+                    status = False
+
+        msg = [self.image_id, "({}, {})".format(state, self.size_h)]
+
+        return " ".join(msg), status
 
     def _parse_snapshots(self, snapshots):
         self.snapshots = ["{name} ({size})".format(name=s['name'],
@@ -738,6 +790,7 @@ class Disk(UINode):
         if not config:
             return
         self._apply_config(config['disks'][self.image_id])
+        self._apply_status()
 
     def _get_meta_data_tcmu(self):
         """
@@ -1003,15 +1056,15 @@ class TargetDisks(UIGroup):
         self.target_iqn = self.parent.name
 
     def load(self, disks):
-        for disk in disks:
-            TargetDisk(self, disk)
+        for image_id, image in disks.items():
+            TargetDisk(self, image_id, image['lun_id'])
 
-    def ui_command_add(self, disk):
-        self.add_disk(disk)
+    def ui_command_add(self, disk, lun_id=None):
+        self.add_disk(disk, lun_id)
 
-    def add_disk(self, disk, success_msg='ok'):
+    def add_disk(self, disk, lun_id, success_msg='ok'):
         rc = 0
-        api_vars = {"disk": disk}
+        api_vars = {"disk": disk, "lun_id": lun_id}
         targetdisk_api = ('{}://localhost:{}/api/'
                           'targetlun/{}'.format(self.http_mode,
                                                 settings.config.api_port,
@@ -1026,7 +1079,9 @@ class TargetDisks(UIGroup):
             disk.owner = owner
             self.logger.debug("- Disk '{}' owner updated to {}"
                               .format(disk.image_id, owner))
-            TargetDisk(self, disk.image_id)
+            target_config = config['targets'][self.target_iqn]
+            lun_id = target_config['disks'][disk.image_id]['lun_id']
+            TargetDisk(self, disk.image_id, lun_id)
             self.logger.debug("- TargetDisk '{}' added".format(disk.image_id))
             if success_msg:
                 self.logger.info(success_msg)
@@ -1075,11 +1130,12 @@ class TargetDisk(UINode):
 
     display_attributes = ['name', 'owner']
 
-    def __init__(self, parent, name):
+    def __init__(self, parent, name, lun_id):
         UINode.__init__(self, name, parent)
         ui_root = self.get_ui_root()
         disk = ui_root.disks.disk_lookup[name]
         self.owner = disk.owner
+        self.lun_id = lun_id
 
     def summary(self):
-        return "Owner: {}".format(self.owner), True
+        return "Owner: {}, Lun: {}".format(self.owner, self.lun_id), True
